import json
from typing import Text, Any, Tuple


def api_response_handler(response, url: Text, expected_status_code: int) -> Tuple[bool, Any]:
    """
    :param response: The response object returned by the requests library API call
    :param url: The URL which was called
    :param expected_status_code: The 2xx code which is expected by the API call to be considered a successful response
    :return: An appropriately formatted error string (error case) or the body of the response (successful case)
    """

    if response.status_code >= 500:
        return False, '[!] [{}] Server Error'.format(response.status_code)
    elif response.status_code == 404:
        return False, '[!] [{}] URL not found: [{}]'.format(response.status_code, url)
    elif response.status_code == 403:
        return False, '[!] [{}] Unauthorized Access'.format(response.status_code)
    elif response.status_code == 401:
        return False, '[!] [{}] Authentication Failed'.format(response.status_code)
    elif response.status_code >= 400:
        return False, '[!] [{}] Bad Request'.format(response.status_code)
    elif response.status_code >= 300:
        return False, '[!] [{}] Unexpected redirect.'.format(response.status_code)
    elif response.status_code != expected_status_code:
        return False, '[?] Unexpected Error: [HTTP {}]: Content: {}'.format(
            response.status_code, response.content)

    return True, json.loads(response.content) if response.content else None



import requests
from typing import List, Tuple, Text, Union
from lethai.utils import api_response_handler


def gender_tag_dataset(username: Text, api_token: Text, dataset: List) -> Union[List[Tuple[Text, Text]], None]:
    """
    :param api_token: API token to access this function
    :param username: Username associated with the API token
    :param dataset: List of words from a dataset. It should not contains any comma(','), fullstops('.') or
    any other kind of delimiter.
    :return: List of tuples of word and gender tag (f - "female", m - "male", n - "neutral")
    """
    __api_url = 'https://dev.api.lethical.ai/v1/feature/gender-tag-dataset'
    __headers = {
        'Content-Type': 'application/json',
        'Auth-Key': api_token,
        'Auth-Username': username
    }
    __json_data = {
        "dataset": dataset
    }
    success, data = api_response_handler(
        response=requests.post(url=__api_url, headers=__headers, json=__json_data),
        url=__api_url,
        expected_status_code=200
    )
    if not success:
        print(data)
        return None

    return list(map(lambda x: (x[0], x[1]), data))



import os
import webbrowser

import json
import requests
from typing import Callable, Text, Optional, Union
from algoauditor.utils import api_response_handler


class config:
    def __init__(self, username: Text, api_token: Text) -> None:
        self.__username = username
        self.__headers = {
            'Content-Type': 'application/json',
            'Auth-Key': api_token,
            'Auth-Username': self.__username
        }
        self.__dataset_api_url = 'https://dev.api.lethical.ai/v1/discrimination/nlg/dataset'
        self.__detect_api_url = 'https://dev.api.lethical.ai/v1/discrimination/nlg/detect'
        self.__dataset = None

    def check_discrimination(self, generator: Callable, model_name: Optional[Text] = None) -> None:
        """
        :param generator: Function with string input and returns the prediction of the NLG model as string
        :param model_name: Optional - String name for the model being run.
        :return:
        """
        # Gets the dataset from the backend if not already available in the frontend
        self.__dataset = self.__get_dataset()
        if not self.__dataset:
            return
        print("Dataset retrieved")

        # Generated predictions with their ML model's generator function
        print("Generating predictions...")
        results = self.__generate_output(generator)
        print("Predictions generated")

        # Now results has the generated text from the NLG model for various categories
        # We need to sync this output with the backend and process the biases (if any) in this model
        json_data = dict()
        json_data["data"] = results
        json_data["model_name"] = model_name

        print("Running analysis on predictions...")
        success, data = api_response_handler(
            response=requests.post(url=self.__detect_api_url, headers=self.__headers, json=json_data),
            url=self.__detect_api_url,
            expected_status_code=200
        )

        if not success:
            print("Analysis failed!")
            print(data)
        else:
            print("Analysis completed. Opening browser to show the analysis...")
            config.__open_browser(data)

    def __get_dataset(self) -> Union[dict, None]:
        """
        :return: The input dataset to provide to generator function to get the predictions from user model
        """
        if self.__dataset:
            return self.__dataset
        success, data = api_response_handler(
            response=requests.get(url=self.__dataset_api_url, headers=self.__headers),
            url=self.__dataset_api_url,
            expected_status_code=200
        )
        if not success:
            print(data)
            return None
        return data

    def __generate_output(self, generator: Callable) -> dict:
        """
        :param generator: Function with string input and returns the prediction of the NLG model as string
        :return: Formatted dictionary of predictions runs on the NLG model using the company's input dataset
        """
        # Create an empty dictionary and fill it in the required format
        data = dict()
        for category in self.__dataset:
            data[category] = dict()
            for datapoint in self.__dataset[category]:
                data[category][datapoint] = dict()
                for text in self.__dataset[category][datapoint]:
                    data[category][datapoint][text] = list()
                    # The range can be increased later for better results
                    # To do so we need to make this function faster
                    # Due to current function implementation the range is kept as small as possible
                    # A possible improvement can be to run the generator function async
                    for i in range(1):
                        data[category][datapoint][text].append(generator(text))

        return data

    @staticmethod
    def __open_browser(results) -> None:
        # Update the json file which is used by the html file
        with open(os.path.join(os.path.abspath(os.path.dirname(__file__)), 'public', 'nlg_json.js'), mode='w') as f:
            f.write('var data = {}'.format(json.dumps(results, indent=2)))

        # Change path to reflect file location
        html_file_path = os.path.join(os.path.abspath(
            os.path.dirname(__file__)), 'public', 'nlg.html')
        webbrowser.open_new_tab(html_file_path)


import json
import requests
from typing import Text, Optional, List
from algoauditor.utils import api_response_handler


class config:
    def __init__(self, username: Text, api_token: Text) -> None:
        self.__username = username
        self.__headers = {
            'Content-Type': 'application/json',
            'Auth-Key': api_token,
            'Auth-Username': self.__username
        }
        self.__detect_api_url = 'https://dev.api.lethical.ai/v1/discrimination/nlp/detect'
        self.__dataset = None

    def check_sentiment_gender_bias(
            self,
            predictions: List,
            intensities: Optional[List] = None,
            model_name: Optional[Text] = None
    ) -> None:
        """
        NOTE: Do not randomize the EEC dataset. We assume the predictions are in the same order
        :param predictions: List of sentiments predicted on EEC dataset
        :param intensities: Optional - List of intensities predicted on EEC dataset
        :param model_name: Optional - String name for the model being run.
        :return: None
        """

        json_data = dict()
        json_data["predictions"] = predictions
        json_data["intensities"] = intensities
        json_data["model_name"] = model_name

        print("Running analysis on predictions...")
        success, data = api_response_handler(
            response=requests.post(url=self.__detect_api_url, headers=self.__headers, json=json_data),
            url=self.__detect_api_url,
            expected_status_code=200
        )

        if not success:
            print("Analysis failed!")
            print(data)
        else:
            print("Analysis completed. Printing results\n")
            print(json.dumps(data, indent=4))
            print("In the next update, this will be added to the dashboard and will be saved in the DB")



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['algoauditor', 'algoauditor.utils']

package_data = \
{'': ['*'], 'algoauditor': ['public/*']}

install_requires = \
['numpy>=1.19.4,<2.0.0', 'requests>=2.25.0,<3.0.0']

setup_kwargs = {
    'name': 'algoauditor',
    'version': '0.0.1',
    'description': 'Lethical AI python library',
    'long_description': '# Lethai Library\nThe library, named "lethai", provides a standard library to interact with the services provided by Lethical.ai.\n\n\n## Getting Started\n\n### Generate API Key\nSteps to follow\n\n  - Create an account on Lethical.ai\n  - Click on API Keys on the side panel\n  - Click "add key" button\n    - Provide tags to the key if needed (They will help you identify and filter the keys for your reference)\n    - Choose the access level for the key\n    - Click on generate\n  - A key will be displayed on the screen please copy it and store it safely for future usage.\n\n*NOTE: The key will only be shown once. This is for security purposes*\n\n### Installation\n> pip3 install lethai\n\n### Usage\nThe various features provided by our library include the following:\n\n- Discrimination detection\n  -[NLG models](./docs/discrimination.md)\n',
    'author': 'Amanl04',
    'author_email': 'amanlodha423@outlook.com',
    'maintainer': 'Aman Lodha',
    'maintainer_email': 'aman@lethical.ai',
    'url': 'https://lethical.ai',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'python_requires': '>=3.6,<4.0',
}


setup(**setup_kwargs)
